"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const os = __importStar(require("os"));
const sauce_json_reporter_1 = require("@saucelabs/sauce-json-reporter");
const cucumber_1 = require("@cucumber/cucumber");
const testcomposer_1 = require("@saucelabs/testcomposer");
const stream_1 = __importDefault(require("stream"));
const isAccountSet = () => {
    return process.env.SAUCE_USERNAME && process.env.SAUCE_ACCESS_KEY;
};
class SauceReporter extends cucumber_1.SummaryFormatter {
    testRun;
    name;
    /**
     * @deprecated Use `name` instead.
     */
    suiteName;
    browserName;
    build;
    tags;
    region;
    outputFile;
    shouldUpload;
    assets;
    testComposer;
    cucumberVersion;
    startedAt;
    endedAt;
    videoStartTime;
    consoleLog;
    passed;
    baseLog;
    logWrapper(buffer) {
        this.baseLog(buffer);
        this.consoleLog.push(Buffer.from(buffer).toString('utf8'));
    }
    constructor(config) {
        super(config);
        const reporterConfig = config.parsedArgvOptions;
        this.baseLog = this.log;
        this.log = this.logWrapper;
        this.suiteName = reporterConfig?.suiteName || '';
        this.name = reporterConfig?.name || '';
        if (this.suiteName && !this.name) {
            this.name = this.suiteName;
        }
        this.browserName = reporterConfig?.browserName || 'chrome';
        this.build = reporterConfig?.build || '';
        this.tags = reporterConfig?.tags || [];
        this.region = reporterConfig?.region || 'us-west-1';
        this.outputFile = reporterConfig?.outputFile || 'sauce-test-report.json';
        this.cucumberVersion = 'unknown';
        this.testRun = new sauce_json_reporter_1.TestRun();
        this.assets = [];
        this.consoleLog = [];
        this.passed = true;
        this.startedAt = new Date().toISOString();
        this.shouldUpload = reporterConfig?.upload !== false;
        // Skip uploads on sauce VM or when credentials aren't set.
        if (process.env.SAUCE_VM || !isAccountSet()) {
            this.shouldUpload = false;
        }
        if (isAccountSet()) {
            this.setupTestComposer();
        }
        if (process.env.SAUCE_VIDEO_START_TIME) {
            this.videoStartTime = new Date(process.env.SAUCE_VIDEO_START_TIME).getTime();
        }
        config.eventBroadcaster?.on('envelope', async (envelope) => {
            if (envelope.testCaseFinished) {
                this.logTestCase(envelope.testCaseFinished);
            }
            if (envelope.testRunFinished) {
                this.endedAt = new Date().toISOString();
                await this.logTestRun(envelope.testRunFinished);
            }
        });
    }
    setupTestComposer() {
        let reporterVersion = 'unknown';
        try {
            const packageData = JSON.parse(fs.readFileSync(path.join(__dirname, '..', 'package.json'), 'utf-8'));
            reporterVersion = packageData.version;
        }
        catch (e) {
            /* empty */
        }
        this.testComposer = new testcomposer_1.TestComposer({
            region: this.region,
            username: process.env.SAUCE_USERNAME || '',
            accessKey: process.env.SAUCE_ACCESS_KEY || '',
            headers: { 'User-Agent': `cucumber-reporter/${reporterVersion}` },
        });
    }
    logTestCase(testCaseFinished) {
        const endTime = new Date();
        const testCaseAttempt = this.eventDataCollector.getTestCaseAttempt(testCaseFinished.testCaseStartedId);
        let parsed;
        try {
            parsed = cucumber_1.formatterHelpers.parseTestCaseAttempt({
                snippetBuilder: this.snippetBuilder,
                supportCodeLibrary: this.supportCodeLibrary,
                testCaseAttempt,
            });
        }
        catch (e) {
            console.error('failed to parse test data: ', e);
            if (e instanceof TypeError) {
                console.warn('\n\n`paths` field might be set multiple times in your configuration file. Please check your Cucumber.js config file.\n');
            }
            return;
        }
        const suite = new sauce_json_reporter_1.Suite(parsed.testCase?.sourceLocation?.uri || '');
        const curr = new sauce_json_reporter_1.Suite(parsed.testCase?.name);
        curr.metadata = {
            attempt: parsed.testCase?.attempt,
            sourceLocation: parsed.testCase?.sourceLocation,
        };
        parsed.testSteps.forEach((testStep) => {
            const test = new sauce_json_reporter_1.Test(`${testStep.keyword}${testStep.text || ''}`);
            const testStatus = testStep.result?.status?.toLowerCase();
            test.status =
                testStatus === 'skipped'
                    ? sauce_json_reporter_1.Status.Skipped
                    : testStatus === 'passed'
                        ? sauce_json_reporter_1.Status.Passed
                        : sauce_json_reporter_1.Status.Failed;
            test.output = testStep.result?.message;
            test.duration = this.durationToMilliseconds(testStep.result?.duration);
            test.attachments = [];
            testStep.attachments.forEach((attachment) => {
                const r = new stream_1.default.Readable();
                r.push(attachment.body);
                r.push(null);
                this.assets.push({
                    filename: attachment.testCaseStartedId + '.log',
                    data: r,
                });
                test.attachments?.push({
                    name: attachment.testCaseStartedId + '.log',
                    contentType: attachment.mediaType,
                    path: '',
                });
            });
            curr.addTest(test);
        });
        this.calculateTestTiming(curr, endTime);
        suite.addSuite(curr);
        this.testRun.addSuite(suite);
    }
    // Calculate each test's startTime and videoTimestamp.
    calculateTestTiming(suite, endTime) {
        let currEndTime = endTime;
        for (let i = suite.tests.length - 1; i >= 0; i--) {
            const test = suite.tests[i];
            const startTime = new Date(currEndTime.getTime() - test.duration);
            suite.tests[i].startTime = startTime;
            if (this.videoStartTime) {
                suite.tests[i].videoTimestamp =
                    (startTime.getTime() - this.videoStartTime) / 1000;
            }
            currEndTime = test.startTime;
        }
    }
    async logTestRun(testRunFinished) {
        this.testRun.status = this.testRun.computeStatus();
        this.passed = testRunFinished.success;
        this.reportToFile(this.testRun);
        if (!this.shouldUpload) {
            if (!process.env.SAUCE_VM && !isAccountSet()) {
                console.warn('Credentials not set! SAUCE_USERNAME and SAUCE_ACCESS_KEY environment ' +
                    'variables must be defined in order for reports to be uploaded to Sauce Labs.');
            }
            return;
        }
        try {
            const job = await this.reportToSauce();
            this.log(`\nReport created: ${job?.url}\n`);
        }
        catch (e) {
            if (e instanceof Error) {
                this.log(`\nFailed to report to Sauce Labs: ${e.message}\n`);
            }
        }
    }
    durationToMilliseconds(duration) {
        return Math.round(duration.seconds * 1000 + duration.nanos / 1000000);
    }
    reportToFile(report) {
        if (!this.outputFile) {
            return;
        }
        fs.mkdirSync(path.dirname(this.outputFile), { recursive: true });
        report.toFile(this.outputFile);
    }
    async reportToSauce() {
        const job = await this.testComposer?.createReport({
            name: this.name,
            startTime: this.startedAt || '',
            endTime: this.endedAt || '',
            framework: 'cucumber',
            frameworkVersion: this.cucumberVersion,
            passed: this.passed,
            tags: this.tags,
            build: this.build,
            browserName: this.browserName,
            browserVersion: '1.0', // Currently there's no reliable way to get the browser version.
            platformName: this.getPlatformName(),
        });
        if (!job || !job.id) {
            return;
        }
        await this.uploadAssets(job.id);
        return job;
    }
    async uploadAssets(jobId) {
        const logReadable = new stream_1.default.Readable();
        logReadable.push(this.consoleLog.join('\n').toString());
        logReadable.push(null);
        const reportReadable = new stream_1.default.Readable();
        reportReadable.push(this.testRun.stringify());
        reportReadable.push(null);
        this.assets.push({
            filename: 'console.log',
            data: logReadable,
        }, {
            filename: 'sauce-test-report.json',
            data: reportReadable,
        });
        await this.testComposer?.uploadAssets(jobId, this.assets).then((resp) => {
            if (resp.errors) {
                for (const err of resp.errors) {
                    this.log(`Failed to upload asset: ${err}\n`);
                }
            }
        }, (e) => this.log(`Failed to upload assets: ${e.message}\n`));
    }
    getPlatformName() {
        switch (os.platform()) {
            case 'darwin':
                return `Mac ${os.release()}`;
            case 'win32':
                return `windows ${os.release()}`;
            case 'linux':
                return 'linux';
            default:
                return 'unknown';
        }
    }
}
exports.default = SauceReporter;
